


/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/


/*=================== D3286 - Error Detector ================================*/
/* LabWindows/CVI 5.0 Instrument Driver										 */
/* Original Release: July/18/2000	( Revision 1.2 )						 */
/* By:				 Rohde & Schwarz, Tektronix, 							 */
/*                   Contact:	ADVANTEST CORPORATION						 */
/*                   			Home Page; http://www.advantest.co.jp		 */
/*                   			e-mail;    info@advantest.co.jp				 */
/*---------------------------------------------------------------------------*/
/* Modification History: 													 */
/*         Revision 1.2 :  July/18/2000										 */
/*																			 */
/*===========================================================================*/
 


#include <formatio.h>
#include <utility.h>
#include <visa.h>
#include <ansi_c.h>
#include "d3286.h"

/*= DEFINES =================================================================*/
#define D3286_REVISION      "Rev 1.2, 7/2000, CVI 5.01" /*  Instrument driver revision */
#define BUFFER_SIZE         512L         				/* File I/O buffer size */
#define TIMEOUT				10000
 
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

static ViString tb_switch_state[2] = {"OF","ON"};
static ViString tb_input_polarity_id[2] = {"MPN","MPI"};
static ViString tb_word_polarity_id[2] = {"WPN","WPI"};
static ViString tb_trigger_output[2] = {"TGCLK","TGPTN"};
static ViString tb_payload_type_id[3] = {"PLW","PLP","CID"};
static ViString tb_frame_polarity_id[2] = {"FPN","FPI"};
static ViString tb_tracking_prefix[4] = {"D","DB","C","CB"};
static ViString tb_terminator_voltage_id[2] = {"GND","M2V"};
static ViString tb_pattern_mode_id[3] = {"PRBS","WORD","FRAME"};
static ViString tb_mark_ratio_id[8] = {"0/8","1/8","1/4","1/2",
									   "8/8","7/8","3/4","1/2B"};
static ViString tb_offset_mode_id[3] = {"OFH","OFM","OFL"};
static ViString tb_meas_fct[6] = {"ERR","ERC","EI","EFI","FRQ","FRC"};
static ViString tb_meas_mode[9] = {"OMI","INS","TOT","OVH","PLD","ALP","SPF","OTH","ALF"};
static ViString tb_err_count_disp_fmt[2] = {"EXP","INT"};
static ViString tb_EI_disp_fmt[2] = {"PCT","ITV"};
static ViString tb_current_data_form[2] = {"PRG","IMD"};
static ViString tb_meas_time_mode[4] = {"NORM","FTIM","FINT","BRST"};
static ViString tb_external_gate[2] = {"GTINT","GTEXT"};
static ViString tb_timer_mode[3] = {"SIN","REP","UNT"};
static ViString tb_time_disp_mode[5] = {"ELP","TMD","PRS","TINT","BMT"};
static ViString tb_file_type[4] = {"S","W","F","M"};

static ViReal64 tb_outp_amplitude_limits[3][2] = { { 0.5, 2.0 }, { 0.6, 1.0 }, { 0.5, 2.0 } };
static ViReal64 tb_outp_offset_limits[3][2] = { { -2.0, 2.0 }, { -1.0, -0.6 }, { -2.0, 2.0 } };

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViStatus CVIFUNC D3286_ibrd (ViSession instrSession, ViByte buffer[],
                              ViInt16 bufsize, ViUInt32 *count);
ViStatus CVIFUNC D3286_ibwrt (ViSession instrSession, ViByte buffer[],
                              ViInt16 bufsize, ViUInt32 *count);
ViBoolean D3286_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean D3286_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean D3286_invalidIntRange (int val, int min, int max);
ViBoolean D3286_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/



/*===========================================================================*/
/* Function: main															 */
/* Purpose:  This is sample program.										 */
/*===========================================================================*/

/*****

void main (void)
{
	ViSession vi, viRM;
    ViStatus status, waitStatus;
    ViUInt32 retCnt = 0L, tmo;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    ViUInt16 stb;
    ViEvent ehnd;
    ViEventType etype;
    double meas;

    if ((status = viOpenDefaultRM(&viRM)) < VI_SUCCESS) {
    	FmtOut("[%i]: the session to the Resource Manager could not be opened.\n", status);
    	return;
    }
    if ((status = viOpen(viRM, "GPIB0::18::INSTR", NULL, 0, &vi)) < VI_SUCCESS) {
    	FmtOut("[%i]: the session to the Device could not be opened.\n", status);
    	return;
    }

	status = D3286_get_measurements (vi, &meas, TIMEOUT);

	status = D3286_get_measurements (vi, &meas, TIMEOUT);

	status = D3286_get_measurements (vi, &meas, TIMEOUT);

	status = D3286_get_measurements (vi, &meas, TIMEOUT);

    if ((status = viGetAttribute (vi, VI_ATTR_TMO_VALUE, &tmo)) >= VI_SUCCESS &&
    	tmo != TIMEOUT) {
	    if ((status = viSetAttribute (vi, VI_ATTR_TMO_VALUE, TIMEOUT)) < VI_SUCCESS) {
	    	FmtOut("[%i]: cannot set timeout to %i ms.\n", status, TIMEOUT);
		    viClose(vi);
		    viClose(viRM);
		    return;
	    }
    } else {
	    viClose(vi);
	    viClose(viRM);
	    return;
    }
    
    Fmt (wrtBuf, "%s<S0");
    total = NumFmtdBytes();
    if ((status = D3286_ibwrt(vi, wrtBuf, total, &retCnt)) < VI_SUCCESS) {
    	FmtOut("[%i]: cannot set status byte mask.\n", status);
        return;
    }
        
    Fmt (wrtBuf, "%s<MS 1");
    total = NumFmtdBytes();
    if ((status = D3286_ibwrt(vi, wrtBuf, total, &retCnt)) < VI_SUCCESS) {
    	FmtOut("[%i]: cannot set status byte mask.\n", status);
        return;
    }
        
    Fmt (wrtBuf, "%s<MES");
    total = NumFmtdBytes();
    if ((status = D3286_ibwrt(vi, wrtBuf, total, &retCnt)) < VI_SUCCESS) {
    	FmtOut("[%i]: cannot set measurement mode.\n", status);
        return;
    }
        
    if ((status = viEnableEvent(vi, VI_EVENT_SERVICE_REQ, VI_QUEUE, VI_NULL)) < VI_SUCCESS) {
    	FmtOut("[%i]: cannot enable service request event.\n", status);
    	return;
    }
    
    if ((waitStatus = viWaitOnEvent(vi, VI_EVENT_SERVICE_REQ, TIMEOUT, &etype, &ehnd)) < VI_SUCCESS)
    	FmtOut("[%i]: error on wait for service request event.\n", waitStatus);
    else {
	    total = BUFFER_SIZE-1;
    	if ((status = viReadSTB(vi, &stb)) >= VI_SUCCESS && (stb & 1) == 1 &&
	        (status = D3286_ibrd(vi, wrtBuf, total, &retCnt)) < VI_SUCCESS)
	    	FmtOut("[%i]: error on read.  STB=%i\n", status, stb);
	    else
	   		FmtOut("%s\n", wrtBuf);
    }
    
    // only close event handle if one was generated
    if (waitStatus >= VI_SUCCESS && viClose(ehnd) >= VI_SUCCESS) {
	    if ((status = viDisableEvent(vi, VI_EVENT_SERVICE_REQ, VI_QUEUE)) < VI_SUCCESS)
	    	FmtOut("[%i]: cannot set close the event.\n", status);
    }
        
    if ((status = viSetAttribute (vi, VI_ATTR_TMO_VALUE, tmo)) < VI_SUCCESS)
    	FmtOut("[%i]: cannot set timeout to %i ms.\n", status, tmo);

    Fmt (wrtBuf, "%s<RTL?");
    total = NumFmtdBytes();
    if ((status = D3286_ibwrt(vi, wrtBuf, total, &retCnt)) >= VI_SUCCESS &&
   	    (status = D3286_ibrd(vi, wrtBuf, BUFFER_SIZE-1, &retCnt)) >= VI_SUCCESS)
   		FmtOut("%s\n", wrtBuf);

	status = D3286_get_measurements (vi, &meas, TIMEOUT);

    viClose(vi);
    viClose(viRM);
}

*****/

/*===========================================================================*/
/* Function: D3286_configure_and_measure                                     */
/* Purpose:  This is sample program.										 */
/*===========================================================================*/

void CVIFUNC D3286_configure_and_measure (void)
{
	ViSession vi, viRM;
    ViStatus status, waitStatus;
    ViUInt32 retCnt = 0L, tmo;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    ViUInt16 stb;
    ViEvent ehnd;
    ViEventType etype;
    double meas;

    if ((status = viOpenDefaultRM(&viRM)) < VI_SUCCESS) {
    	FmtOut("[%i]: the session to the Resource Manager could not be opened.\n", status);
    	return;
    }
    if ((status = viOpen(viRM, "GPIB0::18::INSTR", NULL, 0, &vi)) < VI_SUCCESS) {
    	FmtOut("[%i]: the session to the Device could not be opened.\n", status);
    	return;
    }

	status = D3286_get_measurements (vi, &meas, TIMEOUT);

	status = D3286_get_measurements (vi, &meas, TIMEOUT);

	status = D3286_get_measurements (vi, &meas, TIMEOUT);

	status = D3286_get_measurements (vi, &meas, TIMEOUT);

    if ((status = viGetAttribute (vi, VI_ATTR_TMO_VALUE, &tmo)) >= VI_SUCCESS &&
    	tmo != TIMEOUT) {
	    if ((status = viSetAttribute (vi, VI_ATTR_TMO_VALUE, TIMEOUT)) < VI_SUCCESS) {
	    	FmtOut("[%i]: cannot set timeout to %i ms.\n", status, TIMEOUT);
		    viClose(vi);
		    viClose(viRM);
		    return;
	    }
    } else {
	    viClose(vi);
	    viClose(viRM);
	    return;
    }
    
    Fmt (wrtBuf, "%s<S0");
    total = NumFmtdBytes();
    if ((status = D3286_ibwrt(vi, wrtBuf, total, &retCnt)) < VI_SUCCESS) {
    	FmtOut("[%i]: cannot set status byte mask.\n", status);
        return;
    }
        
    Fmt (wrtBuf, "%s<MS 1");
    total = NumFmtdBytes();
    if ((status = D3286_ibwrt(vi, wrtBuf, total, &retCnt)) < VI_SUCCESS) {
    	FmtOut("[%i]: cannot set status byte mask.\n", status);
        return;
    }
        
    Fmt (wrtBuf, "%s<MES");
    total = NumFmtdBytes();
    if ((status = D3286_ibwrt(vi, wrtBuf, total, &retCnt)) < VI_SUCCESS) {
    	FmtOut("[%i]: cannot set measurement mode.\n", status);
        return;
    }
        
    if ((status = viEnableEvent(vi, VI_EVENT_SERVICE_REQ, VI_QUEUE, VI_NULL)) < VI_SUCCESS) {
    	FmtOut("[%i]: cannot enable service request event.\n", status);
    	return;
    }
    
    if ((waitStatus = viWaitOnEvent(vi, VI_EVENT_SERVICE_REQ, TIMEOUT, &etype, &ehnd)) < VI_SUCCESS)
    	FmtOut("[%i]: error on wait for service request event.\n", waitStatus);
    else {
	    total = BUFFER_SIZE-1;
    	if ((status = viReadSTB(vi, &stb)) >= VI_SUCCESS && (stb & 1) == 1 &&
	        (status = D3286_ibrd(vi, wrtBuf, total, &retCnt)) < VI_SUCCESS)
	    	FmtOut("[%i]: error on read.  STB=%i\n", status, stb);
	    else
	   		FmtOut("%s\n", wrtBuf);
    }
    
    // only close event handle if one was generated
    if (waitStatus >= VI_SUCCESS && viClose(ehnd) >= VI_SUCCESS) {
	    if ((status = viDisableEvent(vi, VI_EVENT_SERVICE_REQ, VI_QUEUE)) < VI_SUCCESS)
	    	FmtOut("[%i]: cannot set close the event.\n", status);
    }
        
    if ((status = viSetAttribute (vi, VI_ATTR_TMO_VALUE, tmo)) < VI_SUCCESS)
    	FmtOut("[%i]: cannot set timeout to %i ms.\n", status, tmo);

    Fmt (wrtBuf, "%s<RTL?");
    total = NumFmtdBytes();
    if ((status = D3286_ibwrt(vi, wrtBuf, total, &retCnt)) >= VI_SUCCESS &&
   	    (status = D3286_ibrd(vi, wrtBuf, BUFFER_SIZE-1, &retCnt)) >= VI_SUCCESS)
   		FmtOut("%s\n", wrtBuf);

	status = D3286_get_measurements (vi, &meas, TIMEOUT);

    viClose(vi);
    viClose(viRM);
}

void CVIFUNC D3286_send_pattern (ViUInt32 frame_number, ViUInt32 row_number,
                                 ViUInt32 byte_number, ViUInt32 frame_byte_count)
{
	ViSession viG, viT, viRM;
    ViStatus status, waitStatus;
    ViUInt32 i, retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    ViUInt16 stb;
    ViEvent ehnd;
    ViEventType etype;
    unsigned char *frame_pattern;
    double scale_fact;
    unsigned int seed;

    if ((status = viOpenDefaultRM(&viRM)) < VI_SUCCESS) {
    	FmtOut("[%i]: the session to the Resource Manager could not be opened.\n", status);
    	return;
    }
    if ((status = viOpen(viRM, "GPIB0::8::INSTR", NULL, 0, &viG)) < VI_SUCCESS) {
    	FmtOut("[%i]: the session to the Device could not be opened.\n", status);
    	return;
    }
    if ((status = viOpen(viRM, "GPIB0::20::INSTR", NULL, 0, &viT)) < VI_SUCCESS) {
    	FmtOut("[%i]: the session to the Device could not be opened.\n", status);
    	return;
    }
	frame_pattern = (unsigned char *)malloc(frame_byte_count);
	seed = (unsigned int)(Timer() * 1000.); 
	srand(seed);
	scale_fact = 256. / (RAND_MAX+1.);
	for (i = 0; i < frame_byte_count; i++)
		frame_pattern[i] = (int)((double)rand() * scale_fact);

    if ((status = D3286_send_frame_pattern (viG, frame_number, row_number, byte_number,
											frame_byte_count, frame_pattern)) < VI_SUCCESS) {
    	FmtOut("[%i]: cannot send frame pattern to pulse generator.\n", status);
    }
    if ((status = D3286_send_frame_pattern (viT, frame_number, row_number, byte_number,
											frame_byte_count, frame_pattern)) < VI_SUCCESS) {
    	FmtOut("[%i]: cannot send frame pattern to BERT.\n", status);
    }

    free(frame_pattern);
    viClose(viT);
    viClose(viG);
    viClose(viRM);
}

/*===========================================================================*/
/* Function: D3286_configure_data_output                                     */
/* Purpose:  This function configures data output.                           */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_data_output (ViSession instrSession,
                                              int terminator_voltage_id,
                                              int input_polarity_id,
                                              ViReal64 input_threshold_level)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (terminator_voltage_id, 1,
							   sizeof(tb_terminator_voltage_id)/sizeof(*tb_terminator_voltage_id)))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidIntRange (input_polarity_id, 1,
							   sizeof(tb_input_polarity_id)/sizeof(*tb_input_polarity_id)))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidViReal64Range (input_threshold_level,
						tb_outp_amplitude_limits[terminator_voltage_id - 1][0],
						tb_outp_amplitude_limits[terminator_voltage_id - 1][1]))
        return VI_ERROR_PARAMETER4;

	/* terminator setting */
    Fmt (wrtBuf, "%s<D%s", tb_terminator_voltage_id[terminator_voltage_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;

    /* input polarity */
    Fmt (wrtBuf, "%s<%s", tb_input_polarity_id[input_polarity_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;

    /* input threshold level */
    Fmt (wrtBuf, "%s<TLVL %f[p2]", input_threshold_level);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_clock_output_voltage                            */
/* Purpose:  This function configures the clock output voltage.              */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_clock_output_voltage (ViSession instrSession,
                                                       int terminator_voltage_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (terminator_voltage_id, 1,
							   sizeof(tb_terminator_voltage_id)/sizeof(*tb_terminator_voltage_id)))
        return VI_ERROR_PARAMETER2;

	/* terminator setting */
    Fmt (wrtBuf, "%s<C%s", tb_terminator_voltage_id[terminator_voltage_id - 1]);
    total = NumFmtdBytes();

    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_clock_delay                                           */
/* Purpose:  This function sets the clock delay.                             */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_clock_delay (ViSession instrSession,
                                        ViInt32 clock_delay_ps)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViInt32Range (clock_delay_ps, -400, 400))
        return VI_ERROR_PARAMETER2;
	
	Fmt (wrtBuf, "%s<DLY %i", clock_delay_ps);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_execute_auto_search                                       */
/* Purpose:  This function executes an auto search.                          */
/*===========================================================================*/
ViStatus CVIFUNC D3286_execute_auto_search (ViSession instrSession)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	Fmt (wrtBuf, "%s<SRHGO");
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_pattern_mode                                          */
/* Purpose:  This function sets the pattern mode.                            */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_pattern_mode (ViSession instrSession, int pattern_mode_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (pattern_mode_id, 1,
							   sizeof(tb_pattern_mode_id)/sizeof(*tb_pattern_mode_id)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_pattern_mode_id[pattern_mode_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_prbs_count_n                                          */
/* Purpose:  This function sets the step count N of pseudo random pattern.   */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_prbs_count_n (ViSession instrSession, ViUInt32 count_n)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if ( count_n != 7 &&
		 count_n != 9 &&
		 count_n != 10 &&
		 count_n != 11 &&
		 count_n != 15 &&
		 count_n != 23 &&
		 count_n != 31 )
        return VI_ERROR_PARAMETER2;
	
	Fmt (wrtBuf, "%s<PB %i[w2p0],0", count_n);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_mark_ratio                                            */
/* Purpose:  This function sets the mark ratio.                              */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_mark_ratio (ViSession instrSession, int mark_ratio_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (mark_ratio_id, 1,
							   sizeof(tb_mark_ratio_id)/sizeof(*tb_mark_ratio_id)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<MR %s", tb_mark_ratio_id[mark_ratio_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_trigger_output                                  */
/* Purpose:  This function sets trigger output.                              */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_trigger_output (ViSession instrSession,
                                                 int trigger_type_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (trigger_type_id, 1,
							   sizeof(tb_trigger_output)/sizeof(*tb_trigger_output)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_trigger_output[trigger_type_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_pattern_trigger_address                               */
/* Purpose:  This function sets trigger address of currently selected        */
/*           pattern mode.                                                   */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_pattern_trigger_address (ViSession instrSession,
                                                    ViUInt32 trigger_address)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	Fmt (wrtBuf, "%s<ADR %i", trigger_address);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_prbs_pattern_trigger_addr                             */
/* Purpose:  This function sets trigger address of pseudo random pattern.    */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_prbs_pattern_trigger_addr (ViSession instrSession,
                                                      ViUInt32 prbs_trigger_address)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (prbs_trigger_address, 0, 134217727))
        return VI_ERROR_PARAMETER2;
	
	Fmt (wrtBuf, "%s<PBTAD %i", prbs_trigger_address);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_alternate_mode                                        */
/* Purpose:  This function sets alternate mode.                              */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_alternate_mode (ViSession instrSession,
                                           int alternate_mode_state)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (alternate_mode_state, 0, 1))
        return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<ALT%s", tb_switch_state[alternate_mode_state]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_select_alternate_pattern                                  */
/* Purpose:  This function sets alternate pattern.                           */
/*===========================================================================*/
ViStatus CVIFUNC D3286_select_alternate_pattern (ViSession instrSession,
                                                 int alternate_pattern_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (alternate_pattern_id, 1, 2))
        return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<ALT%c", 'A' + alternate_pattern_id - 1);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_word_pattern_length                                   */
/* Purpose:  This function sets word pattern length (bit count).             */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_word_pattern_length (ViSession instrSession,
                                                ViUInt32 word_pattern_length)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (word_pattern_length, 1, 8388608))
        return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<BL %i", word_pattern_length);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_word_pattern_trigger_addr                             */
/* Purpose:  This function sets word pattern trigger address.                */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_word_pattern_trigger_addr (ViSession instrSession,
                                                      ViUInt32 word_pattern_trigger_address)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (word_pattern_trigger_address, 0, 524287))
        return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<WDTAD %i", word_pattern_trigger_address);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_word_sync                                             */
/* Purpose:  This function sets word hunting pattern position.               */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_word_sync (ViSession instrSession,
                                      ViUInt32 word_address, ViUInt32 bit_length)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (word_address, 1, 4096))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (bit_length, 1, 16))
        return VI_ERROR_PARAMETER3;
	
    Fmt (wrtBuf, "%s<WDSW %i,%i", word_address, bit_length);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_word_mask_field                                       */
/* Purpose:  This function sets mask field range of word pattern.            */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_word_mask_field (ViSession instrSession,
                                            ViUInt32 word_mask_field_start_address,
                                            ViUInt32 start_bit_number,
                                            ViUInt32 word_mask_field_end_address,
                                            ViUInt32 end_bit_number)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (word_mask_field_start_address, 0, 524287))
		return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (start_bit_number, 1, 16))
		return VI_ERROR_PARAMETER3;
	if (D3286_invalidViUInt32Range (word_mask_field_end_address, 1, 524287))
		return VI_ERROR_PARAMETER4;
	if (D3286_invalidViUInt32Range (end_bit_number, 1, 16))
		return VI_ERROR_PARAMETER5;
	
	Fmt (wrtBuf, "%s<WDMSST %i,%i", word_mask_field_start_address,
									start_bit_number);
	total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;

	Fmt (wrtBuf, "%s<WDMSEN %i,%i", word_mask_field_end_address,
									end_bit_number);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_word_specific_field                                   */
/* Purpose:  This function sets specific field range of word pattern.        */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_word_specific_field (ViSession instrSession,
                                                ViUInt32 word_mask_field_start_address,
                                                ViUInt32 start_bit_number,
                                                ViUInt32 word_mask_field_end_address,
                                                ViUInt32 end_bit_number)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (word_mask_field_start_address, 0, 524287))
		return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (start_bit_number, 1, 16))
		return VI_ERROR_PARAMETER3;
	if (D3286_invalidViUInt32Range (word_mask_field_end_address, 1, 524287))
		return VI_ERROR_PARAMETER4;
	if (D3286_invalidViUInt32Range (end_bit_number, 1, 16))
		return VI_ERROR_PARAMETER5;
	
	Fmt (wrtBuf, "%s<WDSPST %i,%i", word_mask_field_start_address,
									start_bit_number);
	total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;

	Fmt (wrtBuf, "%s<WDSPEN %i,%i", word_mask_field_end_address,
									end_bit_number);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_send_word_pattern                                         */
/* Purpose:  This function sends word pattern.                               */
/*===========================================================================*/
ViStatus CVIFUNC D3286_send_word_pattern (ViSession instrSession,
                                          ViUInt32 word_address,
                                          ViUInt32 word_byte_count,
                                          unsigned char words_sent[])
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (word_address, 0, 524287))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (word_byte_count, 1, 1048576))
        return VI_ERROR_PARAMETER3;
	if (!words_sent)
        return VI_ERROR_PARAMETER4;
	
    Fmt (wrtBuf, "%s<BIN %i,%i", word_address, word_byte_count);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
    if ((D3286_status = D3286_ibwrt (instrSession, words_sent, word_byte_count,
    							 &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_send_word_alternate_pattern                               */
/* Purpose:  This function sends word alternate pattern                      */
/*===========================================================================*/
ViStatus CVIFUNC D3286_send_word_alternate_pattern (ViSession instrSession,
                                                    int word_alternate_pattern_id,
                                                    ViUInt32 word_address,
                                                    ViUInt32 word_byte_count,
                                                    unsigned char words_sent[])
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (word_alternate_pattern_id, 1, 2))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (word_address, 0, 524287))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidViUInt32Range (word_byte_count, 1, 1048576))
        return VI_ERROR_PARAMETER4;
	if (!words_sent)
        return VI_ERROR_PARAMETER5;
	
    Fmt (wrtBuf, "%s<W%cBIN %i,%i", 'A' + word_alternate_pattern_id - 1,
    								word_address, word_byte_count);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
    if ((D3286_status = D3286_ibwrt (instrSession, words_sent, word_byte_count,
    							 &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_polarity                                              */
/* Purpose:  This function sets word pattern polarity.                       */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_polarity (ViSession instrSession, int polarity_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (polarity_id, 1,
							   sizeof(tb_word_polarity_id)/sizeof(*tb_word_polarity_id)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_word_polarity_id[polarity_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_payload_type                                          */
/* Purpose:  This function sets payload type.                                */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_payload_type (ViSession instrSession,
                                         int payload_type_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (payload_type_id, 1,
							   sizeof(tb_payload_type_id)/sizeof(*tb_payload_type_id)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_payload_type_id[payload_type_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_frame_structure                                       */
/* Purpose:  This function sets pattern configuration.                       */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_frame_structure (ViSession instrSession,
                                            ViUInt32 frame_count,
                                            ViUInt32 row_count,
                                            ViUInt32 byte_count_per_row,
                                            ViUInt32 overhead_byte_count,
                                            ViUInt32 cid_bit_count)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (frame_count, 1, 8192))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (row_count, 1, 16))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidViUInt32Range (byte_count_per_row, 40, 32768))
        return VI_ERROR_PARAMETER4;
	if (D3286_invalidViUInt32Range (overhead_byte_count, 1,
													byte_count_per_row - 4))
        return VI_ERROR_PARAMETER5;
	if (D3286_invalidViUInt32Range (cid_bit_count, 0,
						(byte_count_per_row - overhead_byte_count) * 8 - 1))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<FRSTR %i,%i,%i,%i,%i", frame_count, row_count,
				 byte_count_per_row, overhead_byte_count, cid_bit_count);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_frame_pattern_trigger                                 */
/* Purpose:  This function sets trigger position of frame pattern.           */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_frame_pattern_trigger (ViSession instrSession,
                                                  ViUInt32 frame_number,
                                                  ViUInt32 row_number,
                                                  ViUInt32 byte_number)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (frame_number, 1, 8192))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (row_number, 1, 16))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidViUInt32Range (byte_number, 1, 32767))
        return VI_ERROR_PARAMETER4;

    Fmt (wrtBuf, "%s<FRT %i,%i,%i", frame_number, row_number, byte_number);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_frame_sync_word                                       */
/* Purpose:  This function sets frame hunting pattern position.              */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_frame_sync_word (ViSession instrSession,
                                            ViUInt32 frame_number,
                                            ViUInt32 row_number,
                                            ViUInt32 byte_number,
                                            ViUInt32 bit_number)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (frame_number, 1, 4096))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (row_number, 1, 16))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidViUInt32Range (byte_number, 1, 32768))
        return VI_ERROR_PARAMETER4;
	if (D3286_invalidViUInt32Range (bit_number, 1, 1048576))
        return VI_ERROR_PARAMETER5;
	
    Fmt (wrtBuf, "%s<FRSW %i,%i,%i,%i", frame_number, row_number,
    									 byte_number, bit_number);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_frame_mask_field                                      */
/* Purpose:  This function sets mask field range of frame pattern.           */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_frame_mask_field (ViSession instrSession,
                                             ViUInt32 start_frame_number,
                                             ViUInt32 start_row_number,
                                             ViUInt32 start_byte_number,
                                             ViUInt32 start_bit_number,
                                             ViUInt32 end_frame_number,
                                             ViUInt32 end_row_number,
                                             ViUInt32 end_byte_number,
                                             ViUInt32 end_bit_number)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (start_frame_number, 1, 9999))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (start_row_number, 1, 99))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidViUInt32Range (start_byte_number, 1, 32768))
        return VI_ERROR_PARAMETER4;
	if (D3286_invalidViUInt32Range (start_bit_number, 1, 8))
        return VI_ERROR_PARAMETER5;
	if (D3286_invalidViUInt32Range (end_frame_number, 1, 8192))
        return VI_ERROR_PARAMETER6;
	if (D3286_invalidViUInt32Range (end_row_number, 1, 16))
        return VI_ERROR_PARAMETER7;
	if (D3286_invalidViUInt32Range (end_byte_number, 1, 32768))
        return VI_ERROR_PARAMETER8;
	if (D3286_invalidViUInt32Range (end_bit_number, 1, 8))
        return VI_ERROR_PARAMETER9;
	
    Fmt (wrtBuf, "%s<FRMSST %i,%i,%i,%i",
    							start_frame_number, start_row_number,
    							start_byte_number, start_bit_number);
   	total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;

    if (start_frame_number < 9999 && start_row_number < 99) {
	    Fmt (wrtBuf, "%s<FRMSEN %i,%i,%i,%i",
	    						end_frame_number, end_row_number,
    							end_byte_number, end_bit_number);
    	total = NumFmtdBytes();
	    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
	        return D3286_status;
    }
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_frame_specific_field                                  */
/* Purpose:  This function sets specific field range of frame pattern.       */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_frame_specific_field (ViSession instrSession,
                                                 ViUInt32 start_frame_number,
                                                 ViUInt32 start_row_number,
                                                 ViUInt32 start_byte_number,
                                                 ViUInt32 start_bit_number,
                                                 ViUInt32 end_frame_number,
                                                 ViUInt32 end_row_number,
                                                 ViUInt32 end_byte_number,
                                                 ViUInt32 end_bit_number)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (start_frame_number, 1, 9999))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (start_row_number, 1, 99))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidViUInt32Range (start_byte_number, 1, 32768))
        return VI_ERROR_PARAMETER4;
	if (D3286_invalidViUInt32Range (start_bit_number, 1, 8))
        return VI_ERROR_PARAMETER5;
	if (D3286_invalidViUInt32Range (end_frame_number, 1, 8192))
        return VI_ERROR_PARAMETER6;
	if (D3286_invalidViUInt32Range (end_row_number, 1, 16))
        return VI_ERROR_PARAMETER7;
	if (D3286_invalidViUInt32Range (end_byte_number, 1, 32768))
        return VI_ERROR_PARAMETER8;
	if (D3286_invalidViUInt32Range (end_bit_number, 1, 8))
        return VI_ERROR_PARAMETER9;
	
    Fmt (wrtBuf, "%s<FRSPST %i,%i,%i,%i",
    							start_frame_number, start_row_number,
    							start_byte_number, start_bit_number);
   	total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
    if (start_frame_number < 9999 && start_row_number < 99) {
	    Fmt (wrtBuf, "%s<FRSPEN %i,%i,%i,%i",
    							end_frame_number, end_row_number,
   								end_byte_number, end_bit_number);
    	total = NumFmtdBytes();
		if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
	        return D3286_status;
    }
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_send_frame_pattern                                        */
/* Purpose:  This function sends frame pattern.                              */
/*===========================================================================*/
ViStatus CVIFUNC D3286_send_frame_pattern (ViSession instrSession,
                                           ViUInt32 frame_number,
                                           ViUInt32 row_number,
                                           ViUInt32 byte_number,
                                           ViUInt32 frame_byte_count,
                                           unsigned char frame_pattern_sent[])
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidViUInt32Range (frame_number, 1, 8192))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (row_number, 1, 16))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidViUInt32Range (byte_number, 1, 32768))
        return VI_ERROR_PARAMETER4;
	if (D3286_invalidViUInt32Range (frame_byte_count, 1, 1048576))
        return VI_ERROR_PARAMETER5;
	if (!frame_pattern_sent)
        return VI_ERROR_PARAMETER6;
	
    Fmt (wrtBuf, "%s<FBIN %i,%i,%i,%i", frame_number, row_number,
    									byte_number, frame_byte_count);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
    D3286_status = D3286_ibwrt (instrSession, frame_pattern_sent,
										  frame_byte_count, &retCnt);
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_send_frame_alternate_pattern                              */
/* Purpose:  This function sends frame alternate pattern.                    */
/*===========================================================================*/
ViStatus CVIFUNC D3286_send_frame_alternate_pattern (ViSession instrSession,
                                                     int frame_alternate_pattern_id,
                                                     ViUInt32 frame_number,
                                                     ViUInt32 row_number,
                                                     ViUInt32 byte_number,
                                                     ViUInt32 frame_byte_count,
                                                     unsigned char frame_pattern_sent[])
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (frame_alternate_pattern_id, 0, 1))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (frame_number, 1, 4096))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidViUInt32Range (row_number, 1, 16))
        return VI_ERROR_PARAMETER4;
	if (D3286_invalidViUInt32Range (byte_number, 1, 32768))
        return VI_ERROR_PARAMETER5;
	if (D3286_invalidViUInt32Range (frame_byte_count, 1, 1048576))
        return VI_ERROR_PARAMETER6;
	if (!frame_pattern_sent)
        return VI_ERROR_PARAMETER7;
	
    Fmt (wrtBuf, "%s<F%cBIN %i,%i,%i,%i",
    					frame_alternate_pattern_id ? 'B' : 'A',
						frame_number, row_number,
    					byte_number, frame_byte_count);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
    if ((D3286_status = D3286_ibwrt (instrSession, frame_pattern_sent,
    							 frame_byte_count, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_frame_polarity                                        */
/* Purpose:  This function sets frame pattern polarity.                      */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_frame_polarity (ViSession instrSession,
                                           int polarity_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (polarity_id, 1,
							   sizeof(tb_frame_polarity_id)/sizeof(*tb_frame_polarity_id)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_frame_polarity_id[polarity_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_measurement_function                                  */
/* Purpose:  This function sets the measurement function.                    */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_measurement_function (ViSession instrSession,
                                                 int meas_fct_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (meas_fct_id, 1, sizeof(tb_meas_fct)/sizeof(*tb_meas_fct)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_meas_fct[meas_fct_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_measurement_mode                                      */
/* Purpose:  This function sets the measurement mode.                        */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_measurement_mode (ViSession instrSession,
                                             int meas_mode_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (meas_mode_id, 1, sizeof(tb_meas_mode)/sizeof(*tb_meas_mode)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_meas_mode[meas_mode_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_error_count_display_format                            */
/* Purpose:  This function sets the error count measurement display format.  */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_error_count_display_format (ViSession instrSession,
                                                       int format_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (format_id, 1,
							   sizeof(tb_err_count_disp_fmt)/sizeof(*tb_err_count_disp_fmt)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_err_count_disp_fmt[format_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_error_interval_disp_format                            */
/* Purpose:  This function sets the display format of error interval and     */
/*           error-free interval.                                            */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_error_interval_disp_format (ViSession instrSession,
                                                       int format_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (format_id, 1, sizeof(tb_EI_disp_fmt)/sizeof(*tb_EI_disp_fmt)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_EI_disp_fmt[format_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_current_display                                 */
/* Purpose:  This function sets the interim measurement result display		 */
/*           format.                                                         */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_current_display (ViSession instrSession,
                                                  int curr_data_state)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (curr_data_state, 0,
							   sizeof(tb_switch_state)/sizeof(*tb_switch_state) - 1))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<CD%s", tb_switch_state[curr_data_state]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_current_data_mode                                     */
/* Purpose:  This function sets the interim measurement result display		 */
/*           formats of error rate, error count and frame count.             */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_current_data_mode (ViSession instrSession,
                                              int meas_horizon_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (meas_horizon_id, 1,
							   sizeof(tb_current_data_form)/sizeof(*tb_current_data_form)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_current_data_form[meas_horizon_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_measurement_time                                */
/* Purpose:  This function sets the measurement time mode.           		 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_measurement_time (ViSession instrSession,
                                                   int meas_time_mode_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (meas_time_mode_id, 1,
							   sizeof(tb_meas_time_mode)/sizeof(*tb_meas_time_mode)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_meas_time_mode[meas_time_mode_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_select_gate                                               */
/* Purpose:  This function sets the measurement gate control.          		 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_select_gate (ViSession instrSession, int gate_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (gate_id, 1, sizeof(tb_external_gate)/sizeof(*tb_external_gate)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_external_gate[gate_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_start_measurements                                        */
/* Purpose:  This function starts or resets measurement.            		 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_start_measurements (ViSession instrSession)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
    Fmt (wrtBuf, "%s<STT");
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_stop_measurements                                         */
/* Purpose:  This function stops measurement.            					 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_stop_measurements (ViSession instrSession)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
    Fmt (wrtBuf, "%s<STP");
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_auto_sync                                       */
/* Purpose:  This function sets the auto pattern sync function.				 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_auto_sync (ViSession instrSession,
                                            int auto_sync_state_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (auto_sync_state_id, 0,
							   sizeof(tb_switch_state)/sizeof(*tb_switch_state) - 1))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<AS%s", tb_switch_state[auto_sync_state_id]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_frame_sync_for_word                             */
/* Purpose:  This function sets the FRAME SYNC function to be used for the   */
/*           word pattern.													 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_frame_sync_for_word (ViSession instrSession,
                                                      int sync_state_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (sync_state_id, 0,
							   sizeof(tb_switch_state)/sizeof(*tb_switch_state) - 1))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<WS%s", tb_switch_state[sync_state_id]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_frame_sync_for_frame                            */
/* Purpose:  This function sets the FRAME SYNC function to be used for the   */
/*           frame pattern.													 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_frame_sync_for_frame (ViSession instrSession,
                                                       int sync_state_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (sync_state_id, 0,
							   sizeof(tb_switch_state)/sizeof(*tb_switch_state) - 1))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<FS%s", tb_switch_state[sync_state_id]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_execute_resync                                            */
/* Purpose:  This function executes pattern resync.                          */
/*===========================================================================*/
ViStatus CVIFUNC D3286_execute_resync (ViSession instrSession)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
    Fmt (wrtBuf, "%s<SYN");
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_data_buzzer									 */
/* Purpose:  This function sets the bit error detection buzzer.              */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_data_buzzer (ViSession instrSession,
                                              int buzzer_state_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (buzzer_state_id, 0,
							   sizeof(tb_switch_state)/sizeof(*tb_switch_state) - 1))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<BZ%s", tb_switch_state[buzzer_state_id]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_alarm_buzzer									 */
/* Purpose:  This function sets the alarm occurrence buzzer.				 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_alarm_buzzer (ViSession instrSession,
                                               int buzzer_state_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (buzzer_state_id, 0,
							   sizeof(tb_switch_state)/sizeof(*tb_switch_state) - 1))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<BA%s", tb_switch_state[buzzer_state_id]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_timer_mode                                            */
/* Purpose:  This function sets timer mode.                                  */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_timer_mode (ViSession instrSession, int timer_mode_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (timer_mode_id, 1,
							   sizeof(tb_timer_mode)/sizeof(*tb_timer_mode)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_timer_mode[timer_mode_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_set_time_display_mode                                     */
/* Purpose:  This function sets timer display mode.                          */
/*===========================================================================*/
ViStatus CVIFUNC D3286_set_time_display_mode (ViSession instrSession,
                                              int time_display_mode_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (time_display_mode_id, 1,
							   sizeof(tb_time_disp_mode)/sizeof(*tb_time_disp_mode)))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tb_time_disp_mode[time_display_mode_id - 1]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_meas_auto_print                                 */
/* Purpose:  This function enables/disables the auto print mode of			 */
/*           measurement results.											 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_meas_auto_print (ViSession instrSession,
                                                  int auto_print_state_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (auto_print_state_id, 0,
							   sizeof(tb_switch_state)/sizeof(*tb_switch_state) - 1))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<PRI%s", tb_switch_state[auto_print_state_id]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_print_measurements										 */
/* Purpose:  This function temporarily prints measurement results.			 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_print_measurements (ViSession instrSession)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
    Fmt (wrtBuf, "%s<MPRT");
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_configure_error_auto_print								 */
/* Purpose:  This function enables/disables alarm printing.					 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_configure_error_auto_print (ViSession instrSession,
                                                   int auto_print_state_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (auto_print_state_id, 0,
							   sizeof(tb_switch_state)/sizeof(*tb_switch_state) - 1))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<EPR%s", tb_switch_state[auto_print_state_id]);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_printer_paper_feed										 */
/* Purpose:  This function issues printer paper feed.						 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_printer_paper_feed (ViSession instrSession)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
    Fmt (wrtBuf, "%s<FEED");
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_load_file												 */
/* Purpose:  This function loads a file.									 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_load_file (ViSession instrSession, int file_type_id,
                                  ViUInt32 file_number, int alternate_pattern_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (file_type_id, 1, sizeof(tb_file_type)/sizeof(*tb_file_type)))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (file_number, 0, 99))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidIntRange (alternate_pattern_id, 1, 2))
        return VI_ERROR_PARAMETER4;
	
	Fmt (wrtBuf, "%s<LOAD %s,%i,%c", tb_file_type[file_type_id - 1], file_number,
									 'A' + alternate_pattern_id - 1);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_save_file												 */
/* Purpose:  This function saves a file.									 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_save_file (ViSession instrSession, int file_type_id,
                                  ViUInt32 file_number, int alternate_pattern_id)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 total = 0;
    
	if (D3286_invalidIntRange (file_type_id, 1, sizeof(tb_file_type)/sizeof(*tb_file_type)))
        return VI_ERROR_PARAMETER2;
	if (D3286_invalidViUInt32Range (file_number, 0, 99))
        return VI_ERROR_PARAMETER3;
	if (D3286_invalidIntRange (alternate_pattern_id, 1, 2))
        return VI_ERROR_PARAMETER4;
	
	Fmt (wrtBuf, "%s<SAVE %s,%i,%c", tb_file_type[file_type_id - 1], file_number,
									 'A' + alternate_pattern_id - 1);
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_get_measurements											 */
/* Purpose:  This function gets measurement response data.					 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_get_measurements (ViSession instrSession,
                                         ViReal64 *meas_result,
                                         ViUInt32 timeout_ms)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE], hdr[16];
    ViInt16 total = 0;
    double tm, ttm;
    ViUInt16 stb;
    
	if (!meas_result)
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<MES");
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;
        
    tm = Timer() + (double)timeout_ms / 1000.;
    do {
    	D3286_status = viReadSTB(instrSession, &stb);
    	ttm = Timer();
    } while (D3286_status >= VI_SUCCESS && (stb & 1) == 0 && ttm < tm);
    if (D3286_status < VI_SUCCESS)
    	return D3286_status;
    if ((stb & 1) == 0)
    	return VI_ERROR_TMO;

	total = BUFFER_SIZE-1;
    if ((D3286_status = D3286_ibrd (instrSession, wrtBuf, total, &retCnt)) < 0)
	    return D3286_status;
	Scan(wrtBuf, "%s>%s %f", hdr, meas_result);
    
    return D3286_status;        
}

/*===========================================================================*/
/* Function: D3286_get_time													 */
/* Purpose:  This function gets time response data.							 */
/*===========================================================================*/
ViStatus CVIFUNC D3286_get_time (ViSession instrSession, ViChar meas_time[],
                                 ViUInt32 timeout_ms)
{
    ViStatus D3286_status = VI_SUCCESS;
    ViUInt32 retCnt = 0L;
    ViByte wrtBuf[BUFFER_SIZE], hdr[16];
    ViInt16 total = 0;
    double tm, ttm;
    ViUInt16 stb;
    
	if (!meas_time)
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<TIM");
    total = NumFmtdBytes();
    if ((D3286_status = D3286_ibwrt (instrSession, wrtBuf, total, &retCnt)) < 0)
        return D3286_status;

    tm = Timer() + (double)timeout_ms / 1000.;
    do {
    	D3286_status = viReadSTB(instrSession, &stb);
    	ttm = Timer();
    } while (D3286_status >= VI_SUCCESS && (stb & 1) == 0 && ttm < tm);
    if (D3286_status < VI_SUCCESS)
    	return D3286_status;
    if ((stb & 1) == 0)
    	return VI_ERROR_TMO;

	total = BUFFER_SIZE-1;
    if ((D3286_status = D3286_ibrd (instrSession, wrtBuf, total, &retCnt)) < 0)
	    return D3286_status;
	Scan(wrtBuf, "%s>%s %s", hdr, meas_time);
    
    return D3286_status;        
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: D3286_ibwrt                                                     */
/* Purpose:  Wrapper for viWrite.                                            */
/*===========================================================================*/
ViStatus CVIFUNC D3286_ibwrt (ViSession instrSession, ViByte buffer[],
                              ViInt16 bufsize, ViUInt32 *count)
{
    if (instrSession)
    	return viWrite (instrSession, buffer, bufsize, count);
    else {
        FmtOut ("%[%i%]: \042%s\042\n", bufsize, buffer);
        return VI_SUCCESS;        
    }
}

/*===========================================================================*/
/* Function: D3286_ibrd                                                      */
/* Purpose:  Wrapper for viRead.                                             */
/*===========================================================================*/
ViStatus CVIFUNC D3286_ibrd (ViSession instrSession, ViByte buffer[],
                              ViInt16 bufsize, ViUInt32 *count)
{
    return instrSession ? viRead(instrSession, buffer, bufsize, count) : -1;
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean D3286_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean D3286_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Integer Value Out Of Range - int                                */
/* Purpose:  This function checks an integer value to see if it              */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean D3286_invalidIntRange (int val, int min, int max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return        */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean D3286_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
